import React, { useRef, useEffect, useState } from "react";
import PropTypes from "prop-types";
import { Link, Collapse } from "@material-ui/core";

const OverflowText = (props) => {
  const { displayHtmlText, showMoreText, showLessText } = props;
  // Define state and function to update the value
  const [showMore, setShowMore] = useState(true);
  const [showLess, setShowLess] = useState(false);

  const textElementRef = useRef();
  const oneLineHeight = 24;

  const compareSize = () => {
    const textHasWrapped = textElementRef.current.scrollHeight > oneLineHeight;

    setShowMore(textHasWrapped);
  };

  useEffect(() => {
    setShowLess(false);
    setShowMore(true);

    compareSize();
  }, [props.displayHtmlText]);

  useEffect(() => {
    window.addEventListener("resize", compareSize);

    return () => {
      window.removeEventListener("resize", compareSize);
    };
  }, []);

  const toggleExpanded = () => {
    setShowMore(!showMore);
    setShowLess(!showLess);
  };

  return (
    <div className="OverflowText">
      <Collapse in={showLess} collapsedHeight={oneLineHeight}>
        <div
          ref={textElementRef}
          dangerouslySetInnerHTML={{ __html: displayHtmlText }}
        />
      </Collapse>
      {(showMore || showLess) && (
        <Link component="button" variant="body2" onClick={toggleExpanded}>
          {(showMore && showMoreText) || (showLess && showLessText)}
        </Link>
      )}
    </div>
  );
};

export default OverflowText;

OverflowText.defaultProps = {
  displayHtmlText: null
};

OverflowText.propTypes = {
  displayHtmlText: PropTypes.string,
  showLessText: PropTypes.string.isRequired,
  showMoreText: PropTypes.string.isRequired
};
