import randomColor from "randomcolor";
import range from "lodash.range";
import { MIN_FOR_10_PATIENTS_OR_FEWER } from "utilities";


const numberOfColors = 100;
const barDataShape = { labels: [], values: [] };
const colorScheme = range(0, numberOfColors).map(() => randomColor());

export const getBarGraphTickLabel = (label, maxLength = 24) => {
  const trimmedLabel = label.trim();
  return trimmedLabel.length > maxLength
    ? `${trimmedLabel.substring(0, maxLength)}...`
    : trimmedLabel;
};

export const resultsToBarData = (breakdownResults) => {
  return breakdownResults.reduce((barData, currentResult) => {
    const replacementForZeroToDisplayGraphLabel = MIN_FOR_10_PATIENTS_OR_FEWER;
    const { value } = currentResult;
    const labels = [...barData.labels, currentResult.dataKey.trim()];
    const values = [
      ...barData.values,
      value === 0 ? replacementForZeroToDisplayGraphLabel : value,
    ];

    return { labels, values };
  }, barDataShape);
};

export const convertDemographicsToGraphData = (
  graphLabels = [],
  graphValues = []
) => {
  const newColors = colorScheme.slice(0, graphLabels.length);

  return {
    labels: graphLabels,
    datasets: [
      {
        barThickness: "30",
        backgroundColor: newColors,
        borderColor: "#2782a3",
        borderWidth: 1,
        hoverBackgroundColor: "lightgray",
        hoverBorderColor: "#2782a3",
        data: graphValues,
      },
    ],
  };
};
