export const DEFAULT_VALUE = 'unavailable';
export const NO_RESULTS_VALUE = -1;
export const EXPORT_FILENAME = 'export.csv';
export const countValueConverter = ({ count = DEFAULT_VALUE } = {}) => (count === NO_RESULTS_VALUE ? 0 : count);
export const resultValueConverter = ({ value = DEFAULT_VALUE } = {}) => (value === NO_RESULTS_VALUE ? 0 : value);

// Group by breakdown type...i.e. Race, Age, Gender...e.t.c.
export const buildBreakdownSchema = breakdowns => (
  breakdowns.reduce((schema, breakdown) => {
    const { description, results } = breakdown;

    if (!schema[description]) {
      schema[description] = { dataKeys: new Set() };
    }
    results.forEach(({ dataKey }) => {
      schema[description].dataKeys.add(dataKey);
    });
    return schema;
  }, {}));

export const mapQueryToCSV = (queryResult) => {
  const { breakdowns, institutionResults } = queryResult;
  const line1 = `SHRINE QUERY RESULTS (OBFUSCATED PATIENT COUNTS),, ${institutionResults.map(({ institutionName }) => institutionName).join(',')}`;
  const line2 = `\nAll Patients,,${institutionResults.map(countValueConverter).join(',')}`;
  const schema = buildBreakdownSchema(breakdowns);

  // Based on the schema we just built, map the result of each breakdown.
  let breakdownRows = '';
  Object.entries(schema).forEach(([breakdownType, { dataKeys }]) => {
    dataKeys.forEach((dataKey) => {
      const rowTitle = `${breakdownType.trim()} ${dataKey}`;
      const values = breakdowns.filter(({ description }) => description === breakdownType)
        .reduce((line, { results }) => {
          const result = (
            !results
              ? {}
              : results.find(_ => _.dataKey === dataKey)
          );

          return `${line},${resultValueConverter(result)}`;
        }, '');
      breakdownRows = `${breakdownRows},\n${rowTitle},${values}`;
    });
  });
  return `${line1}${line2}${breakdownRows}`;
};
